Package: SpatialExperiment
Version: 1.22.0
Title: S4 Class for Spatially Resolved -omics Data
Description: Defines an S4 class for storing data from spatial -omics experiments. 
    The class extends SingleCellExperiment to 
    support storage and retrieval of additional information from spot-based and 
    molecule-based platforms, including spatial coordinates, images, and 
    image metadata. A specialized constructor function is included for data 
    from the 10x Genomics Visium platform.
Authors@R: c(
    person("Dario", "Righelli", role=c("aut", "cre"), 
        email="dario.righelli@gmail.com",
        comment=c(ORCID="0000-0003-1504-3583")),
    person("Davide", "Risso", role=c("aut"), 
        email="risso.davide@gmail.com",
        comment=c(ORCID="0000-0001-8508-5012")),
    person("Helena L.", "Crowell", role=c("aut"), 
        email="helena.crowell@cnag.eu",
        comment=c(ORCID="0000-0002-4801-1767")), 
    person("Lukas M.", "Weber", role=c("aut"), 
        email="lmweb012@gmail.com",
        comment=c(ORCID="0000-0002-3282-1730")), 
    person("Nicholas J.", "Eagles", role=c("ctb"),
        email="nickeagles77@gmail.com"))
URL: https://github.com/drighelli/SpatialExperiment
BugReports: https://github.com/drighelli/SpatialExperiment/issues
License: GPL-3
Encoding: UTF-8
biocViews: DataRepresentation, DataImport, Infrastructure,
        ImmunoOncology, GeneExpression, Transcriptomics, SingleCell,
        Spatial
Depends: R (>= 4.1.0), methods, SingleCellExperiment
Imports: rjson, grDevices, magick, utils, S4Vectors,
        SummarizedExperiment, BiocGenerics, BiocFileCache
Suggests: knitr, rmarkdown, testthat, BiocStyle, BumpyMatrix,
        DropletUtils, VisiumIO
VignetteBuilder: knitr
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/SpatialExperiment
git_branch: RELEASE_3_23
git_last_commit: 90e092a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:52:41 UTC; biocbuild
Author: Dario Righelli [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1504-3583>),
  Davide Risso [aut] (ORCID: <https://orcid.org/0000-0001-8508-5012>),
  Helena L. Crowell [aut] (ORCID:
    <https://orcid.org/0000-0002-4801-1767>),
  Lukas M. Weber [aut] (ORCID: <https://orcid.org/0000-0002-3282-1730>),
  Nicholas J. Eagles [ctb]
Maintainer: Dario Righelli <dario.righelli@gmail.com>
